﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Configuration;
using System.Xml;
using Microsoft.Win32;
using System.Runtime.InteropServices;
using System.Net;
using System.IO;

namespace prox
{
    class Program
    {
        [DllImport("wininet.dll")]
        public static extern bool InternetSetOption(IntPtr hInternet, int dwOption, IntPtr lpBuffer, int dwBufferLength);
        public const int INTERNET_OPTION_SETTINGS_CHANGED = 39;
        public const int INTERNET_OPTION_REFRESH = 37;

        public static Dictionary<string, string> settings = new Dictionary<string, string>();
        public static bool settings_valid = false;
        public static Dictionary<string, string> derived_settings = new Dictionary<string, string>();


        static void Main(string[] args)
        {

            // because I can never agree on a syntax...
            string[] args_set_list  = { "set",  "/s", "-s", "/set",  "--set"  };
            string[] args_dump_list = { "dump", "/d", "-d", "/dump", "--dump" };
            string[] args_off_list  = { "off",  "/o", "-o", "/off",  "--off"  };
            string[] args_on_list = { "on", "--on", "--on" };
            string[] args_test_list = { "test", "/t", "-t", "/test", "--test" };
            string[] args_gui_list = { "gui", "/g", "-g", "/gui", "--gui" };
            string[] args_parse_list = { "parse", "/p", "-p", "/parse", "--parse" };
            string[] args_list_list = { "list", "/l", "-l", "/list", "--list" };
            string[] args_regedit_list = { "regedit", "/r", "-r", "/regedit", "--regedit", "/reg", "--reg" };

            string program_mode = "null";
            string chosen_configuration = "null";

            if (args.Length == 0)
            {
                Usage();
                About();
                Environment.Exit(1);
            }

            for (int x = 0; x < args.Length; x++)
            {
                if (args_set_list.Contains(args[x]))
                {
                    program_mode = "set";
                    continue;
                }

                if (args_dump_list.Contains(args[x]))
                {
                    program_mode = "dump";
                    continue;
                }

                if (args_off_list.Contains(args[x]))
                {
                    program_mode = "off";
                    continue;
                }

                if (args_test_list.Contains(args[x]))
                {
                    program_mode = "test";
                    continue;
                }

                if (args_gui_list.Contains(args[x]))
                {
                    program_mode = "gui";
                    continue;
                }

                if (args_list_list.Contains(args[x]))
                {
                    program_mode = "list";
                    continue;
                }
                if (args_parse_list.Contains(args[x]))
                {
                    program_mode = "parse";
                    continue;
                }

                if (args_regedit_list.Contains(args[x]))
                {
                    program_mode = "regedit";
                    continue;
                }

                if (args_on_list.Contains(args[x]))
                {
                    program_mode = "on";
                    continue;
                }

                chosen_configuration = args[x];
            }

            if (program_mode == "on")
            {
                program_mode = "set";
                chosen_configuration = "default";
            }

            
            const string userRoot = "HKEY_CURRENT_USER";
            const string subkey = "Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings";
            const string keyName = userRoot + "\\" + subkey;


            switch (program_mode)
            {
                case "set":
                    LoadSettingsFromXML(chosen_configuration);

                    if (!settings_valid)
                    {
                        Console.WriteLine("Error: can't load config for {0}.  Is XML file there and proper?", chosen_configuration);
                        Environment.Exit(1);
                    }

                    DeriveSettingsFromXML();

                    Registry.SetValue(keyName, "ProxyServer", derived_settings["ProxyServer"]);
                    Registry.SetValue(keyName, "ProxyEnable", Convert.ToInt32(derived_settings["ProxyEnable"]));
                    Registry.SetValue(keyName, "ProxyOverride", derived_settings["ProxyOverride"]);
                    Registry.SetValue(keyName, "AutoDetect", Convert.ToInt32(derived_settings["AutoDetect"]));

                    if (derived_settings["AutoConfigURL"] != null ) {
                        Registry.SetValue(keyName, "AutoConfigURL", derived_settings["AutoConfigURL"]);
                    }

                    
                    // delete the ones that null out...
                    using (RegistryKey key = Registry.CurrentUser.OpenSubKey(keyName, true))
                    {
                        if (derived_settings["AutoConfigURL"] == null)
                        {
                            try
                            {
                                key.DeleteValue("AutoConfigURL");
                            }
                            catch
                            {
                                // @@ realistically I should check for key exists before deleting but can't be bothered at right this second.
                            }
                        }
                        // key.DeleteValue("X");
                    }
            
                    // These lines implement the Interface in the beginning of program 
                    // They cause the OS to refresh the settings, causing IP to realy update
                    InternetSetOption(IntPtr.Zero, INTERNET_OPTION_SETTINGS_CHANGED, IntPtr.Zero, 0);
                    InternetSetOption(IntPtr.Zero, INTERNET_OPTION_REFRESH, IntPtr.Zero, 0);

                    break;
                case "parse":

                    Console.WriteLine("\nConfiguration: {0}", chosen_configuration);

                    Console.WriteLine("\nXML config");
                    Console.WriteLine("----------");


                    LoadSettingsFromXML(chosen_configuration);

                    foreach (KeyValuePair<string, string> kvp in settings)
                    {
                        Console.WriteLine("{0,-25}: {1}", kvp.Key, kvp.Value);
                    }

                    DeriveSettingsFromXML();
                    Console.WriteLine("\nDerived config");
                    Console.WriteLine("--------------");
                    foreach (KeyValuePair<string, string> kvp in derived_settings)
                    {
                        Console.WriteLine("{0,-25}: {1}", kvp.Key, kvp.Value);
                    }
                    break;

                case "dump":
                    Console.WriteLine("Registry Dump");
                    Console.WriteLine("-------------");
                    Console.WriteLine(keyName);
                    Console.WriteLine("\n");
                    Console.WriteLine("ProxyServer: {0}", Registry.GetValue(keyName, "ProxyServer", "<unset>"));
                    Console.WriteLine("ProxyEnable: {0}", Registry.GetValue(keyName, "ProxyEnable", "<unset>"));
                    Console.WriteLine("ProxyOverride: {0}", Registry.GetValue(keyName, "ProxyOverride", "<unset>"));
                    Console.WriteLine("AutoDetect: {0}", Registry.GetValue(keyName, "AutoDetect", "<unset>"));
                    Console.WriteLine("AutoConfigURL: {0}", Registry.GetValue(keyName, "AutoConfigURL", "<unset>"));

                    break;

                case "gui":  // open the windows gui at the appropriate position
                    System.Diagnostics.Process.Start("rundll32.exe", "shell32.dll,Control_RunDLL inetcpl.cpl,,4");
                    Environment.Exit(0);
                    break;

                case "regedit":  // open regedit at the appropriate position

                    // cheating is fun!!
                    const string lastkey = "HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Applets\\Regedit";
                    Registry.SetValue(lastkey, "Lastkey", keyName);
                    System.Diagnostics.Process.Start("regedit.exe");
                    Environment.Exit(0);
                    break;

                case "off":  // turn off all the proxy stuff
                    

                    Registry.SetValue(keyName, "ProxyServer", "");
                    Registry.SetValue(keyName, "ProxyEnable", 0);
                    Registry.SetValue(keyName, "ProxyOverride", "");
                    Registry.SetValue(keyName, "AutoDetect", 0);

                    
                    // delete the ones that null out...
                    using (RegistryKey key = Registry.CurrentUser.OpenSubKey(keyName, true))
                    {
                            try
                            {
                                key.DeleteValue("AutoConfigURL");
                            }
                            catch
                            {
                                // @@ realistically I should check for key exists before deleting but can't be bothered at right this second.
                            }
                    }
            



                    // These lines implement the Interface in the beginning of program 
                    // They cause the OS to refresh the settings, causing IP to realy update
                    InternetSetOption(IntPtr.Zero, INTERNET_OPTION_SETTINGS_CHANGED, IntPtr.Zero, 0);
                    InternetSetOption(IntPtr.Zero, INTERNET_OPTION_REFRESH, IntPtr.Zero, 0);

                    break;
                case "list":
                    try
                    {
                        XmlDocument doc = new XmlDocument();
                        doc.Load(AppDomain.CurrentDomain.BaseDirectory + "proxyconfig.xml");

                        XmlNode node = doc.DocumentElement;

                        foreach (XmlNode xnode in node.ChildNodes)
                        {                            
                            Console.WriteLine(xnode.Name);
                        }

                    }
                    catch
                    {
                        Console.WriteLine("Error Loading Settings");
                    }
                    break;
                case "test":

                    if (chosen_configuration == "all")
                    {
                        // loop through them all and do them

                        try
                        {
                            XmlDocument doc = new XmlDocument();
                            doc.Load(AppDomain.CurrentDomain.BaseDirectory + "proxyconfig.xml");

                            XmlNode node = doc.DocumentElement;


                            //Dictionary<string, string> settings = new Dictionary<string, string>();

                            foreach (XmlNode xnode in node.ChildNodes)
                            {
                                //Console.WriteLine(xnode.Name);
                                LoadSettingsFromXML(xnode.Name);
                                Console.WriteLine(xnode.Name + ":");
                                TestProxy(settings["http_server_ip"], settings["http_server_port"]);
                                Console.WriteLine();
                                
                            }

                        }
                        catch
                        {
                            Console.WriteLine("Error Loading Settings");
                        }
                    }
                    else
                    {
                        LoadSettingsFromXML(chosen_configuration);
                        TestProxy(settings["http_server_ip"], settings["http_server_port"]);

                    }
                    break;
            }


            

        }

        static void TestProxy(string ip, string port_s)
        {
            int port = Convert.ToInt32(port_s);
            Stream ReceiveStream = null;
            StreamReader sr = null;
            string url = "http://www.google.com";
            int bytecount = 0;
            long ms = 0;

            Console.WriteLine("Retrieving {0} via {1}:{2}", url, ip, port);

            // from https://msdn.microsoft.com/en-us/library/ms172495%28v=vs.90%29.aspx
            try
            {
                //if (!"".Equals(txtProxy.Text))
                //{
                    WebProxy proxyObject = new WebProxy(ip,port);

                    // Disable proxy use when the host is local.
                    proxyObject.BypassProxyOnLocal = true;

                    // HTTP requests use this proxy information.
                    GlobalProxySelection.Select = proxyObject;

                //}

                System.Diagnostics.Stopwatch STOPWATCH = System.Diagnostics.Stopwatch.StartNew();
                WebRequest req = WebRequest.Create(url);
                WebResponse result = req.GetResponse();
                ReceiveStream = result.GetResponseStream();
                Encoding encode = System.Text.Encoding.GetEncoding("utf-8");
                sr = new StreamReader(ReceiveStream, encode);

                // Read the stream into arrays of 30 characters 
                // to add as items in the list box. Repeat until 
                // buffer is read.
                Char[] read = new Char[30];
                int count = sr.Read(read, 0, 30);
                
                while (count > 0)
                {
                    //String str = new String(read, 0, count);
                    //lstResults.Items.Add(str);
                    //Console.WriteLine(str);
                    bytecount += count;
                    count = sr.Read(read, 0, 30);
                }
                STOPWATCH.Stop();
                ms = STOPWATCH.ElapsedMilliseconds;
            }
            catch (WebException ex)
            {
                string message = ex.Message;
                HttpWebResponse response = (HttpWebResponse)ex.Response;
                if (null != response)
                {
                    message = response.StatusDescription;
                    response.Close();
                }
                //lstResults.Items.Add(message);
                Console.WriteLine(message);
            }
            catch (Exception ex)
            {
                //lstResults.Items.Add(ex.Message);
                Console.WriteLine(ex.Message);
            }
            finally
            {
                try
                {
                    Console.WriteLine("Read {0} bytes in {1} ms.  {2} Kbps", bytecount, ms, bytecount * 8 / ms);
                }
                catch
                {
                    Console.WriteLine("Read {0} bytes in {1} ms.  0 Kbps", bytecount, ms);
                }
                

                try
                {
                    ReceiveStream.Close();
                    sr.Close();
                }
                catch { }
            }
        }

        static void About()
        {
            string about =@"proxy.exe v0.2
Eli Fulkerson 16-Apr-2015
http://www.elifulkerson.com for updates
";

            Console.WriteLine(about);
        }

        static void Usage()
        {
            string usage = @"Usage: proxy [mode] configname

Modes:
    set     -   sets the system proxy to the configuration named 'configname'
                specified in proxyconfig.xml
    off     -   turns off the system proxy
    on      -   alias for 'proxy set default', requires a 'default' in the
                proxyconfig.xml file.
    dump    -   reads the system proxy settings from the registry and dumps to
                console
    list    -   lists the configname sections in proxyconfig.xml
    parse   -   reads proxyconfig.xml and parses the settings, dumping results
                to console   
    
    gui     -   opens 'Internet Properties' -> 'Connections'
    regedit -   opens regedit, starting at the proxy setting key location
    
    test    -   connects to the proxy in 'configname' and tries to retrieve
                'http://www.google.com', reports errors and speed

    test all    special case for test, loops through all options in the config
                file and checks each one.  Don't name any of them 'all'.
";

            Console.WriteLine(usage);
        }



        static void DeriveSettingsFromXML()
        {
            // ProxyServer
            if (settings["use_same_settings_for_all"] == "1")
            {
                derived_settings["ProxyServer"] = settings["http_server_ip"] + ":" + settings["http_server_port"];
            }
            else
            {
                derived_settings["ProxyServer"] =   "http=" + settings["http_server_ip"] + ":" + settings["http_server_port"] + ";" +
                                                    "https=" + settings["https_server_ip"] + ":" + settings["https_server_port"] + ";" +
                                                    "ftp=" + settings["ftp_server_ip"] + ":" + settings["ftp_server_port"] + ";" +
                                                    "socks=" + settings["socks_server_ip"] + ":" + settings["socks_server_port"];
            }

            // ProxyEnable
            derived_settings["ProxyEnable"] = settings["ProxyEnable"];

            // AutoConfigURL
            if (settings["use_AutoConfigURL"] == "1")
            {
                derived_settings["AutoConfigURL"] = settings["AutoConfigURL"];
            }
            else
            {
                derived_settings["AutoConfigURL"] = null;
            }

            // ProxyOverride
            derived_settings["ProxyOverride"] = settings["exclude_prefix_list"];
            if (settings["bypass_local"] == "1")
            {
                if (derived_settings["ProxyOverride"].Length > 0)
                {
                    derived_settings["ProxyOverride"] += ";<local>";
                }
                else
                {
                    derived_settings["ProxyOverride"] = "<local>";
                }
            }

            // AutoDetect
            derived_settings["AutoDetect"] = settings["AutoDetect"];            
        }

        static void LoadSettingsFromXML(string chosen_configuration)
        {
            try
            {
                XmlDocument doc = new XmlDocument();
                doc.Load(AppDomain.CurrentDomain.BaseDirectory + "proxyconfig.xml");

                XmlNode node = doc.DocumentElement.SelectSingleNode(chosen_configuration);


                //Dictionary<string, string> settings = new Dictionary<string, string>();

                foreach (XmlNode xnode in node.ChildNodes)
                {
                    settings[xnode.Attributes["key"].Value] = xnode.Attributes["value"].Value;
                }

                settings_valid = true;
            }
            catch
            {
                Console.WriteLine("Error Loading Settings");
            }
        }
    }
}
